; ADPCM-A and ADPCM-B example thing | 68K portion
;==============================================================================;
; defines
	include "inc/neogeo.inc"		; Neo-Geo HW/BIOS defs
	include "inc/ram_bios.inc"		; BIOS RAM locations
	include "inc/input.inc"			; Input defines
	include "soundcmd.inc"			; shared include with 68K

;==============================================================================;
; begin code
	include "header_68k.inc"

; headers: cart vs. cd chaos
	ifd TARGET_CD
		include "header_cd.inc"
	else
		include "header_cart.inc"
	endif

;==============================================================================;
; and now, the standard boilerplate code (hacked up for the demo, as usual)
;==============================================================================;
; USER
; Needs to perform actions according to the value in BIOS_USER_REQUEST.
; Must jump back to SYSTEM_RETURN at the end so the BIOS can have control.

USER:
	move.b	d0,REG_DIPSW		; kick watchdog
	lea		BIOS_WORKRAM,sp		; set stack pointer to BIOS_WORKRAM
	move.w	#0,LSPC_MODE		; Disable auto-animation, timer interrupts, set auto-anim speed to 0 frames
	move.w	#7,LSPC_IRQ_ACK		; ack. all IRQs

	move.w	#$2000,sr			; Enable VBlank interrupt, go Supervisor

	; Handle user request
	clr.l	d0
	move.b	(BIOS_USER_REQUEST).l,d0
	lsl.b	#2,d0				; shift value left to get offset into table
	lea		cmds_USER_REQUEST,a0
	movea.l	(a0,d0),a0
	jsr		(a0)

;------------------------------------------------------------------------------;
; BIOS_USER_REQUEST commands
cmds_USER_REQUEST:
	dc.l	userReq_StartupInit	; Command 0 (Initialize)
	dc.l	userReq_Eyecatch	; Command 1 (Custom eyecatch)
	dc.l	userReq_Game		; Command 2 (Demo Game/Game)
	dc.l	userReq_Game		; Command 3 (Title Display)

;------------------------------------------------------------------------------;
; userReq_StartupInit
; Initialize the backup work area. This demo doesn't use it.

userReq_StartupInit:
	move.b	d0,REG_DIPSW		; kick watchdog
	jmp		SYSTEM_RETURN

;------------------------------------------------------------------------------;
; userReq_Eyecatch
; Only to be fully coded if your game uses its own eyecatch (value at $114 is 1).
; Otherwise, jmp to SYSTEM_RETURN.

userReq_Eyecatch:
	move.b	d0,REG_DIPSW		; kick watchdog
	jmp		SYSTEM_RETURN

;------------------------------------------------------------------------------;
; userReq_Game
; This is the complex one. For this demo, we're only going to treat it as a
; combination of initialization and main loop, but for a real game, you might
; want to have BIOS_USER_REQUEST commands 2 and 3 do different things.

userReq_Game:
	move.b	d0,REG_DIPSW		; kick watchdog

	; perform initialization
	move.b	#0,curOptionNum
	move.b	#0,prevOptionNum
	move.b	#0,curSampleRate
	move.b	#0,pcmaPlay
	move.b	#0,pcmbPlay
	move.b	#0,pcmaLoop
	move.b	#0,pcmbLoop

	; set up palettes
	move.b	d0,PALETTE_BANK1	; use palette bank 1
	lea		paletteData,a0
	lea		PALETTES,a1
	move.l	#(16*NUM_PALETTES)-1,d7
.ldpal:
	move.w	(a0)+,(a1)+
	dbra	d7,.ldpal

	; background color
	;		  tRGB
	move.w	#$8024,PALETTE_BACKDROP

	jsr		FIX_CLEAR			; clear fix layer, add borders on sides
	jsr		LSP_1st				; clear first sprite

	jsr		DrawInitDisplay

; execution continues into main loop.
;------------------------------------------------------------------------------;
; mainLoop
; The game's main loop. This is where you run the actual game part.

mainLoop:
	move.b	d0,REG_DIPSW		; kick the watchdog

	; do things like:
	jsr		CheckInput			; check inputs
	jsr		WaitVBlank			; wait for the vblank
	; and other things that would normally happen in a game's main loop.

	jmp		mainLoop

;==============================================================================;
; PLAYER_START
; Called by the BIOS if one of the Start buttons is pressed while the player
; has enough credits (or if the the time runs out on the title menu?).
; We're not using this in this demo.

PLAYER_START:
	move.b	d0,REG_DIPSW		; kick the watchdog
	rts

;==============================================================================;
; DEMO_END
; Called by the BIOS when the Select button is pressed; ends the demo early.

DEMO_END:
	; if necessary, store any items in the (MVS) backup RAM.
	rts

;==============================================================================;
; COIN_SOUND
; Called by the BIOS when a coin is inserted; should play a coin drop sound.
; We don't actually do anything here since this isn't meant to take coins.

COIN_SOUND:
	; Send a sound code
	rts

;==============================================================================;
; VBlank
; VBlank interrupt, run things we want to do every frame.

VBlank:
	; check if the BIOS wants to run its vblank
	btst	#7,BIOS_SYSTEM_MODE
	bne		.gamevbl
	; run BIOS vblank
	jmp		SYSTEM_INT1

	;-----------------------;
	; run the game's vblank
.gamevbl
	movem.l d0-d7/a0-a6,-(sp)	; save registers
	move.w	#4,LSPC_IRQ_ACK		; acknowledge the vblank interrupt
	move.b	d0,REG_DIPSW		; kick the watchdog

	; do things in vblank

.endvbl:
	jsr		SYSTEM_IO			; "Call SYSTEM_IO every 1/60 second."
	jsr		MESS_OUT			; Puzzle Bobble calls MESS_OUT just after SYSTEM_IO
	jsr		UpdatePlaybackDisplay

	move.b	#0,flag_VBlank		; clear vblank flag so waitVBlank knows to stop
	movem.l (sp)+,d0-d7/a0-a6	; restore registers
	rte

;==============================================================================;
; IRQ2
; Level 2/timer interrupt, unused here. You could use it for effects, though.

IRQ2:
	move.w	#2,LSPC_IRQ_ACK		; ack. interrupt #2 (HBlank)
	move.b	d0,REG_DIPSW		; kick watchdog
	rte

;==============================================================================;
; IRQ3
; Level 3 IRQ, unused here. Might be used for something else on CD, though.
; (More research needed)

IRQ3:
	move.w  #1,LSPC_IRQ_ACK		; acknowledge interrupt 3
	move.b	d0,REG_DIPSW		; kick watchdog
	rte

;==============================================================================;
; WaitVBlank
; Waits for VBlank to finish (via a flag cleared at the end).

WaitVBlank:
	move.b	#1,flag_VBlank

.waitLoop
	tst.b	flag_VBlank
	bne.s	.waitLoop 

	rts

;==============================================================================;
; freemlib code
	include "freemlib/fix.asm"

;==============================================================================;
; palette data
	include "paldata.inc"

; main interface strings
str_Header1:		dc.b "ADPCM-A & ADPCM-B Example Program",$FF
str_Header2:		dc.b "by freem",$FF,$00
str_WebAddr:		dc.b "http://ajworld.net/neogeodev/",$FF

str_PlayA_Normal:	dc.b "Play A (Unlooped)",$FF
str_PlayB_Normal:	dc.b "Play B (Unlooped)",$FF

str_PlayA_Looped:	dc.b "Play A (Looped)",$FF
str_PlayB_Looped:	dc.b "Play B (Looped)",$FF

str_RateB:			dc.b "Rate B",$FF,$00

str_StopA:			dc.b "Stop A",$FF,$00
str_StopB:			dc.b "Stop B",$FF,$00

;==============================================================================;
; Draw initial display
DrawInitDisplay:
	; header 1
	move.w	#$0303,d0
	move.w	#$13,d1
	lea		str_Header1,a0
	jsr		fix_DrawString

	; header 2
	move.w	#$0304,d0
	move.w	#$13,d1
	lea		str_Header2,a0
	jsr		fix_DrawString

	; web address
	move.w	#$0306,d0
	move.w	#$13,d1
	lea		str_WebAddr,a0
	jsr		fix_DrawString

	; play unlooped
	move.w	#$0C0D,d0
	move.w	#$13,d1
	lea		str_PlayA_Normal,a0
	jsr		fix_DrawString

	move.w	#$0C0E,d0
	move.w	#$13,d1
	lea		str_PlayB_Normal,a0
	jsr		fix_DrawString

	; play looped
	move.w	#$0C10,d0
	move.w	#$13,d1
	lea		str_PlayA_Looped,a0
	jsr		fix_DrawString

	move.w	#$0C11,d0
	move.w	#$13,d1
	lea		str_PlayB_Looped,a0
	jsr		fix_DrawString

	; adpcm-b rate
	move.w	#$0C13,d0
	move.w	#$13,d1
	lea		str_RateB,a0
	jsr		fix_DrawString
	jsr		UpdateRateDisplay

	; stop samples
	move.w	#$0C15,d0
	move.w	#$13,d1
	lea		str_StopA,a0
	jsr		fix_DrawString

	move.w	#$0C16,d0
	move.w	#$13,d1
	lea		str_StopB,a0
	jsr		fix_DrawString

	; adpcm-a and adpcm-b sprite displays
	move.w	#1,LSPC_INCR
	;--------------------------------------;
	; sprite 1: PCMA label 1/2 ($00040)
	; SCB1/tilemap
	move.w	#$0040,LSPC_ADDR
	move.w	#$0040,LSPC_DATA
	move.w	#$0100,LSPC_DATA
	; SCB2/shrink ($8001)
	move.w	#$8001,LSPC_ADDR
	move.w	#$0FFF,LSPC_DATA
	; SCB4/X pos ($8401; 80px)
	move.w	#$8401,LSPC_ADDR
	move.w	#80<<7,LSPC_DATA
	; SCB3/Y pos, Sticky, Sprite size (real Y pos 56px)
	; 496-64 = 432
	move.w	#$8201,LSPC_ADDR
	move.w	#((496-56)<<7)|(0<<6)|1,LSPC_DATA

	; sprite 2: PCMA label 2/2 ($00041)
	; SCB1/tilemap
	move.w	#$0080,LSPC_ADDR
	move.w	#$0041,LSPC_DATA
	move.w	#$0100,LSPC_DATA
	; SCB2/shrink ($8002)
	move.w	#$8002,LSPC_ADDR
	move.w	#$0FFF,LSPC_DATA
	; SCB3/Y pos, Sticky, Sprite size (real Y pos 56px)
	move.w	#$8202,LSPC_ADDR
	move.w	#1<<6,LSPC_DATA

	;--------------------------------------;
	; sprite 3: PCMB label 1/2 ($00040)
	; SCB1/tilemap
	move.w	#$00C0,LSPC_ADDR
	move.w	#$0040,LSPC_DATA
	move.w	#$0100,LSPC_DATA
	; SCB2/shrink ($8003)
	move.w	#$8003,LSPC_ADDR
	move.w	#$0FFF,LSPC_DATA
	; SCB4/X pos ($8403; 176px)
	move.w	#$8403,LSPC_ADDR
	move.w	#176<<7,LSPC_DATA
	; SCB3/Y pos, Sticky, Sprite size (real Y pos 56px)
	move.w	#$8203,LSPC_ADDR
	move.w	#((496-56)<<7)|(0<<6)|1,LSPC_DATA

	; sprite 4: PCMB label 2/2 ($00042)
	; SCB1/tilemap
	move.w	#$0100,LSPC_ADDR
	move.w	#$0042,LSPC_DATA
	move.w	#$0100,LSPC_DATA
	; SCB2/shrink ($8004)
	move.w	#$8004,LSPC_ADDR
	move.w	#$0FFF,LSPC_DATA
	; SCB3/Y pos, Sticky, Sprite size (real Y pos 56px)
	move.w	#$8204,LSPC_ADDR
	move.w	#1<<6,LSPC_DATA

	;--------------------------------------;
	; status and loop display
	jsr		UpdatePlaybackDisplay

	; draw initial cursor
	jmp		UpdateCursor
	; use UpdateCursor's rts instead of throwing our own here

;==============================================================================;
; Check input
CheckInput:
	move.b	BIOS_P1CHANGE,d3
	andi.b	#INPUT_A|INPUT_UDLR,d3
	beq		.CheckInput_end		; don't bother

.CheckInput_A:
	btst	#INBIT_A,d3
	beq		.CheckInput_Left

	; [A button]
	; depends on what item we're on...
	jsr		DoButtonInput

.CheckInput_Left:
	btst	#INBIT_LEFT,d3
	beq		.CheckInput_Right

	; [Left]
	; are we on an option that allows Left/Right? (namely, Rate B)
	move.b	curOptionNum,d1
	cmpi.b	#4,d1
	; don't even bother checking left if we're not on option 4
	bne		.CheckInput_Up

	; pressed left on Rate B
	move.b	curSampleRate,d1
	beq		.CheckInput_Left_Wrap

	; normal left
	subi.b	#1,d1
	move.b	d1,curSampleRate
	jsr		UpdatePCMBRate
	bra		.CheckInput_Up

.CheckInput_Left_Wrap:
	move.b	#7,curSampleRate
	jsr		UpdatePCMBRate
	bra		.CheckInput_Up

.CheckInput_Right:
	btst	#INBIT_RIGHT,d3
	beq		.CheckInput_Up

	; [Right]
	; are we on an option that allows Left/Right? (namely, Rate B)
	move.b	curOptionNum,d1
	cmpi.b	#4,d1
	bne		.CheckInput_Up

	; pressed right on Rate B
	move.b	curSampleRate,d1
	cmpi.b	#7,d1
	beq		.CheckInput_Right_Wrap

	; normal right
	addi.b	#1,d1
	move.b	d1,curSampleRate
	jsr		UpdatePCMBRate
	bra		.CheckInput_Up

.CheckInput_Right_Wrap:
	clr		curSampleRate
	jsr		UpdatePCMBRate
	bra		.CheckInput_Up

.CheckInput_Up:
	btst	#INBIT_UP,d3
	beq		.CheckInput_Down

	; [Up]
	move.b	curOptionNum,prevOptionNum

	move.b	curOptionNum,d1
	bne		.CheckInput_Up_Normal

.CheckInput_Up_Wrap:
	move.b	#6,curOptionNum
	bra		.CheckInput_Up_Update

.CheckInput_Up_Normal:
	subi.b	#1,d1
	move.b	d1,curOptionNum

.CheckInput_Up_Update:
	bra		UpdateCursor

.CheckInput_Down:
	btst	#INBIT_DOWN,d3
	; if we've managed to get here (meaning none of the desired buttons were hit)
	; then what the fuck were we doing here in the first place?
	; "this should never happen", but computing follows "never say never" to a T.
	beq		.CheckInput_end

	; [Down]
	move.b	curOptionNum,prevOptionNum

	move.b	curOptionNum,d1
	cmpi.b	#6,d1
	bne		.CheckInput_Down_Normal

.CheckInput_Down_Wrap:
	clr.b	curOptionNum
	bra		.CheckInput_Down_Update

.CheckInput_Down_Normal:
	addi.b	#1,d1
	move.b	d1,curOptionNum

.CheckInput_Down_Update:
	bra		UpdateCursor

.CheckInput_end
	rts

;==============================================================================;
; rates
; (yes I hardcoded the "Hz" instead of setting it once and forgetting it.
; view the end of the C ROM/SPR file for the explanation.)
str_Rate0:	dc.b "11025Hz",$FF
str_Rate1:	dc.b "16538Hz",$FF
str_Rate2:	dc.b "22050Hz",$FF
str_Rate3:	dc.b "27563Hz",$FF
str_Rate4:	dc.b "33075Hz",$FF
str_Rate5:	dc.b "38588Hz",$FF
str_Rate6:	dc.b "44100Hz",$FF
str_Rate7:	dc.b "55125Hz",$FF

tbl_RateStrings:
	dc.l	str_Rate0
	dc.l	str_Rate1
	dc.l	str_Rate2
	dc.l	str_Rate3
	dc.l	str_Rate4
	dc.l	str_Rate5
	dc.l	str_Rate6
	dc.l	str_Rate7

;------------------------------------------------------------------------------;
UpdateRateDisplay:
	clr		d0
	clr		d1
	move.b	curSampleRate,d0
	lsl.b	#2,d0
	lea		tbl_RateStrings,a0
	move.l	(a0,d0),a0
	move.w	#$1313,d0
	move.w	#$13,d1
	jsr		fix_DrawString
	rts

;------------------------------------------------------------------------------;
UpdatePCMBRate:
	; --update display (d0)--
	jsr		UpdateRateDisplay

	; --send command (d2)--
	clr		d2
	move.b	curSampleRate,d2
	; AND the current sample rate number with 7 (force 0-7)
	andi.b	#$07,d2
	; then OR with $80 since the rate commands start at $80
	or.b	#$80,d2
	; send this command to the Z80
	move.b	d2,REG_SOUND
	jsr		Z80Wait

	rts

;==============================================================================;
; todo: use a function table instead?
; then again this is a demo, and demos have lazy ass code

DoButtonInput:
	; check what item we're on
	move.b	curOptionNum,d0

	; 0: Play A (unlooped)
	beq		.DoButtonInput_0

	; 4: Rate B
	; the rate changes when left/right is pressed, but pressing a button on
	; this option does nothing.
	cmpi.b	#4,d0
	beq		.DoButtonInput_end

	; 1: Play B (unlooped)
	cmpi.b	#1,d0
	beq		.DoButtonInput_1

	; 2: Play A (looped)
	cmpi.b	#2,d0
	beq		.DoButtonInput_2

	; 3: Play B (looped)
	cmpi.b	#3,d0
	beq		.DoButtonInput_3

	; 5: Stop A
	cmpi.b	#5,d0
	beq		.DoButtonInput_5

	; 6: Stop B
	cmpi.b	#6,d0
	beq		.DoButtonInput_6

	; panic handler
	bra		.DoButtonInput_end

.DoButtonInput_0:	; (Play A unlooped)
	move.b	#1,pcmaPlay
	move.b	#0,pcmaLoop
	move.b	#SOUNDCMD_NoLoopA,REG_SOUND
	jsr		Z80Wait
	move.b	#SOUNDCMD_PlayA,REG_SOUND
	jsr		Z80Wait
	bra		.DoButtonInput_end

.DoButtonInput_1:	; (Play B unlooped)
	move.b	#1,pcmbPlay
	move.b	#0,pcmbLoop
	move.b	#SOUNDCMD_NoLoopB,REG_SOUND
	jsr		Z80Wait
	move.b	#SOUNDCMD_PlayB,REG_SOUND
	jsr		Z80Wait
	bra		.DoButtonInput_end

.DoButtonInput_2:	; (Play A looped)
	move.b	#1,pcmaPlay
	move.b	#1,pcmaLoop
	move.b	#SOUNDCMD_LoopA,REG_SOUND
	jsr		Z80Wait
	move.b	#SOUNDCMD_PlayA,REG_SOUND
	jsr		Z80Wait
	bra		.DoButtonInput_end

.DoButtonInput_3:	; (Play B looped)
	move.b	#1,pcmbPlay
	move.b	#1,pcmbLoop
	move.b	#SOUNDCMD_LoopB,REG_SOUND
	jsr		Z80Wait
	move.b	#SOUNDCMD_PlayB,REG_SOUND
	jsr		Z80Wait
	bra		.DoButtonInput_end

.DoButtonInput_5:	; (Stop A)
	move.b	#0,pcmaPlay
	move.b	#SOUNDCMD_StopA,REG_SOUND
	jsr		Z80Wait
	bra		.DoButtonInput_end

.DoButtonInput_6:	; (Stop B)
	move.b	#0,pcmbPlay
	move.b	#SOUNDCMD_StopB,REG_SOUND
	jsr		Z80Wait

.DoButtonInput_end:
	rts

;==============================================================================;
; VRAM locations for cursor
tbl_CursorPos:
	dc.w	$714D		; Play Unlooped A
	dc.w	$714E		; Play Unlooped B
	dc.w	$7150		; Play Looped A
	dc.w	$7151		; Play Looped B
	dc.w	$7153		; Rate B
	dc.w	$7155		; Stop A
	dc.w	$7156		; Stop B

; update cursor location
UpdateCursor:
	; --blank out old cursor--
	clr		d0
	clr		d1
	move.b	prevOptionNum,d0
	lsl.b	#1,d0
	lea		tbl_CursorPos,a0
	move.w	(a0,d0),d1
	; address in d1, draw a blank cursor
	move.w	d1,LSPC_ADDR
	move.w	#$1300,LSPC_DATA

	; --draw new cursor--
	clr		d0
	clr		d1
	move.b	curOptionNum,d0
	lsl.b	#1,d0
	lea		tbl_CursorPos,a0
	move.w	(a0,d0),d1
	; address in d1, draw regular cursor
	move.w	d1,LSPC_ADDR
	move.w	#$1311,LSPC_DATA
	rts

;==============================================================================;
UpdatePlaybackDisplay:
	move.w	#1,LSPC_INCR
	;--------------------------------------;
	; sprite 5: PCMA status display (stop: $00048; play: $00049)
	; SCB1/tilemap
	; tile LSB depends on pcmaPlay
	move.w	#$0140,LSPC_ADDR
	move.w	#$0048,d0
	add.b	pcmaPlay,d0
	move.w	d0,LSPC_DATA
	move.w	#$0100,LSPC_DATA

	; SCB2/shrink ($8005)
	move.w	#$8005,LSPC_ADDR
	move.w	#$0FFF,LSPC_DATA

	; SCB4/X pos ($8405; 112px)
	move.w	#$8405,LSPC_ADDR
	move.w	#112<<7,LSPC_DATA

	; SCB3/Y pos, Sticky, Sprite size (real Y pos 56px)
	move.w	#$8205,LSPC_ADDR
	move.w	#((496-56)<<7)|(0<<6)|1,LSPC_DATA

	;--------------------------------------;
	; sprite 6: PCMB status display (stop: $00048; play: $00049)
	; SCB1/tilemap

	ifd TARGET_CD
		; CD-specific
		move.w	#$0180,LSPC_ADDR
		move.w	#$004E,d0
		move.w	d0,LSPC_DATA
		move.w	#$0100,LSPC_DATA
	else
		; cart-specific
		move.w	#$0180,LSPC_ADDR
		move.w	#$0048,d0
		; tile LSB depends on pcmbPlay
		add.b	pcmbPlay,d0
		move.w	d0,LSPC_DATA
		move.w	#$0100,LSPC_DATA
	endif

	; SCB2/shrink ($8006)
	move.w	#$8006,LSPC_ADDR
	move.w	#$0FFF,LSPC_DATA

	; SCB4/X pos ($8406; 208px)
	move.w	#$8406,LSPC_ADDR
	move.w	#208<<7,LSPC_DATA

	; SCB3/Y pos, Sticky, Sprite size (real Y pos 56px)
	move.w	#$8206,LSPC_ADDR
	move.w	#((496-56)<<7)|(0<<6)|1,LSPC_DATA

	;--------------------------------------;
	; sprite 7: PCMA loop display (yes: $00043; no: $00044)
	; SCB1/tilemap
	; tile LSB depends on pcmaLoop
	move.w	#$01C0,LSPC_ADDR
	move.w	#$0044,d0
	sub.b	pcmaLoop,d0
	move.w	d0,LSPC_DATA
	move.w	#$0100,LSPC_DATA

	; SCB2/shrink ($8007)
	move.w	#$8007,LSPC_ADDR
	move.w	#$0FFF,LSPC_DATA

	; SCB4/X pos ($8407; 128px)
	move.w	#$8407,LSPC_ADDR
	move.w	#128<<7,LSPC_DATA

	; SCB3/Y pos, Sticky, Sprite size (real Y pos 56px)
	move.w	#$8207,LSPC_ADDR
	move.w	#((496-56)<<7)|(0<<6)|1,LSPC_DATA

	;--------------------------------------;
	; sprite 8: PCMB loop display (yes: $00043; no: $00044)
	; SCB1/tilemap

	ifd TARGET_CD
		; CD-specific
		move.w	#$0200,LSPC_ADDR
		move.w	#$004F,d0
		move.w	d0,LSPC_DATA
		move.w	#$0100,LSPC_DATA
	else
		; cart-specific
		move.w	#$0200,LSPC_ADDR
		move.w	#$0044,d0
		; tile LSB depends on pcmbLoop
		sub.b	pcmbLoop,d0
		move.w	d0,LSPC_DATA
		move.w	#$0100,LSPC_DATA
	endif

	; SCB2/shrink ($8008)
	move.w	#$8008,LSPC_ADDR
	move.w	#$0FFF,LSPC_DATA

	; SCB4/X pos ($8406; 224px)
	move.w	#$8408,LSPC_ADDR
	move.w	#224<<7,LSPC_DATA

	; SCB3/Y pos, Sticky, Sprite size (real Y pos 56px)
	move.w	#$8208,LSPC_ADDR
	move.w	#((496-56)<<7)|(0<<6)|1,LSPC_DATA

	rts

;==============================================================================;
; Z80Wait
; Gives the Z80 some time to react. (Taken from smkdan's demo)

Z80Wait:
	move.w	d0,-(sp)
	move.w	#1000,d0		; arbitrary delay length

.Z80Wait_loop:
	dbra	d0,.Z80Wait_loop

	move.w	(sp)+,d0
	rts

;==============================================================================;
; program RAM
	include "ram_user.inc"
