; ADPCM-A and ADPCM-B example thing | Z80 portion
;==============================================================================;
; This is a highly simplified Z80 driver; it only handles playing ADPCM-A and
; ADPCM-B samples.
;==============================================================================;
; defines
	include "sounddef.inc"
	include "sysmacro.inc"
	include "soundcmd.inc" ; shared include with 68K

; Defines for RST usage, in case the locations change later.
rst_Write45		= $18
rst_Write67		= $20
rst_BusyWait	= $28
;==============================================================================;
	org $0000
Start:
	di
	jp		EntryPoint
;==============================================================================;
	org $0018
j_write45:
	jp		write_45
;==============================================================================;
	org $0020
j_write67:
	jp		write_67
;==============================================================================;
	org $0028
; Keep checking the busy flag in Status 0 until it's clear.

; Code from smkdan's example M1 driver (adpcma_demo2/sound_M1.asm), where he
; uses this instead of portWriteDelayPart2 and portWriteDelayPart4.
; It's noted that "MAME doesn't care". The hardware does, however.

CheckBusyFlag:
	in		a,(4)			; read Status 0 (busy flag in bit 7)
	add		a
	jr		C,CheckBusyFlag
	ret
;==============================================================================;
	org $0038
; the IRQ belongs here.
j_IRQ:
	di
	jp		IRQ
;==============================================================================;

;==============================================================================;
	org $0066
; NMI
; Inter-processor communications.

; In this driver, the NMI gets the command from the 68K and interprets it.

NMI:
	; save registers
	push	af
	push	bc
	push	de
	push	hl

	in		a,(0)			; Acknowledge NMI, get command from 68K via Port 0
	ld		(curCommand),a	; update curCommand
	ld		b,a				; load value into b for comparisons

	; "Commands $01 and $03 are always expected to be implemented as they
	; are used by the BIOSes for initialization purposes." - NeoGeoDev Wiki
	cp		1				; Command 1 (Slot Switch)
	jp		Z,doCmd01
	cp		3				; Command 3 (Soft Reset)
	jp		Z,doCmd03
	or		a				; check if Command is 0
	jp		Z,endNMI		; exit if Command 0

	; do NMI crap/handle communication... I'm not fully sure what to do yet.
	call	HandleCommand

	xor		a				; clear a for now.
	out		(0xC),a			; Reply to 68K with something.
	out		(0),a			; Write to port 0 (Clear sound code)

	; update previous command
	ld		a,(curCommand)
	ld		(prevCommand),a
endNMI:
	; restore registers
	pop		hl
	pop		de
	pop		bc
	pop		af
	retn

;==============================================================================;
; IRQ (called from $0038)
; Handle an interrupt request.

IRQ:
; save registers
	push	af
	push	bc
	push	de
	push	hl
	push	ix
	push	iy

	; update internal Status 1 register
	in		a, (6)
	ld		(intStatus1),a

	; --check status of ADPCM channels--

	; for ADPCM-B, looping is handled automatically (via repeat flag).
	; we don't need to check anything. ADPCM-A, on the other hand...

	; in this demo, only ADPCM-A channel 1 (bit 0) is used.
	; only stop/loop the sample if we've reached the end of it.
	ld		a,(intStatus1)
	bit		0,a
	jp		Z,.IRQ_afterPCMA

	; check if we're supposed to loop the sample at all
	ld		a,(loopA)
	cp		1
	jp		NZ,.IRQ_NoLoopPCMA

	; if so, loop sample from beginning
	call	command_PlayPCMA
	jp		.IRQ_afterPCMA

.IRQ_NoLoopPCMA:
	; stop sample on ADPCM-A channel 1
	ld		de,0x0081
	write67

.IRQ_afterPCMA:
	; update internal Status 0 register
	in		a, (4)
	ld		(intStatus0),a

	; check timers, I guess.

.IRQ_end:
	; restore registers
	pop		iy
	pop		ix
	pop		hl
	pop		de
	pop		bc
	pop		af

	ei

	; return
	ret

;==============================================================================;
; EntryPoint
; The entry point for the sound driver.

EntryPoint:
	ld		sp,0xFFFC		; Set stack pointer ($FFFD-$FFFE is used for other purposes)
	im		1				; Set Interrupt Mode 1 (IRQ at $38)
	xor		a				; make value in A = 0

	; Clear RAM at $F800-$FFFF
	ld		(0xF800),a		; set $F800 = 0
	ld		hl,0xF800		; 00 value is at $F800
	ld		de,0xF801		; write sequence begins at $F801
	ld		bc,0x7FF		; end at $FFFF
	ldir					; clear out memory

	; Initialize variables
	ld		(loopA),a
	ld		(loopB),a

	; Silence/Stop SSG, FM, and ADPCM
	call	ssg_Stop
	call	fm_Stop
	; todo: silence ADPCM-A, ADPCM-B
	; "various writes to ports 4/5 and 6/7"

	;-------------------------------------------;
	; write 1 to port $C0 (what is the purpose?)
	ld		a,1
	out		(0xC0),a

	; continue setting up the hardware, etc.

	ld		de,0x2730		; Reset Timer flags, Disable Timer IRQs
	write45					; write to ports 4 and 5
	ld		de,0x1001		; Reset ADPCM-B
	write45					; write to ports 4 and 5
	ld		de,0x1C00		; Unmask ADPCM-A and B flag controls
	write45					; write to ports 4 and 5

	; Initialize more variables
	call	SetDefaultBanks	; Set default program banks

; this section subject to further review
;{
	; set timer values??

	; Start Timers ($27,$3F to ports 4 and 5)
	ld		de,0x273F		; Reset Timer flags, Enable Timer IRQs, Load Timers
	write45					; write to ports 4 and 5
;}

	; Set ADPCM-A shared volume
	ld		de,0x013F		; Set ADPCM-A volume to Maximum
	write67					; write to ports 6 and 7

	; set ADPCM-B volume
	ld		a,0xFF
	ld		(intPCMB_Volume),a		; ADPCM-B volume max
	ld		de,0x1BFF		; Set ADPCM-B volume to Maximum
	write45					; write to ports 4 and 5

	; set ADPCM-B output on Left and Right channels
	ld		a,0xC0
	ld		(intPCMB_LeftRight),a	; ADPCM-B left and right channels
	ld		de,0x11C0		; Set ADPCM-B Left/Right output
	write45					; write to ports 4 and 5

	; set default ADPCM-B sample rate
	ld		a,0xDA
	ld		(intPCMB_SampRateL),a	; Delta-N Sampling Rate (L)
	ld		de,0x19DA		; Set ADPCM-B Sampling Rate LSB
	write45					; write to ports 4 and 5

	ld		a,0x32
	ld		(intPCMB_SampRateH),a	; Delta-N Sampling Rate (H)
	ld		de,0x1A32		; Set ADPCM-B Sampling Rate LSB
	write45					; write to ports 4 and 5

	; (Enable NMIs)
	ld		a,1
	out		(8),a			; Write to Port 8 (Enable NMI)
	ei

;------------------------------------------------------------------------------;
; MainLoop
; Nothing here; everything is handled via NMI (and sometimes IRQ)

MainLoop:
	jp		MainLoop

;==============================================================================;
; write_45
; Writes data (from registers de) to ports 4 and 5. (YM2610 A1 line=0)

write_45:
	push	af
	ld		a,d
	out		(4),a			; write to port 4 (address 1)
	rst		rst_BusyWait	; Write delay 1 (17 cycles)
	ld		a,e
	out		(5),a			; write to port 5 (data 1)
	rst		rst_BusyWait	; Write delay 2 (83 cycles)
	pop		af
	ret

;------------------------------------------------------------------------------;
; write_67
; Writes data (from registers de) to ports 6 and 7. (YM2610 A1 line=1)

write_67:
	push	af
	ld		a,d
	out		(6),a			; write to port 6 (address 2)
	rst		rst_BusyWait	; Write delay 1 (17 cycles)
	ld		a,e
	out		(7),a			; write to port 7 (data 2)
	rst		rst_BusyWait	; Write delay 2 (83 cycles)
	pop		af
	ret

;==============================================================================;
; doCmd01
; Performs setup work for Command $01 (Slot Change).

doCmd01:
	di
	xor		a
	out		(0xC),a			; write 0 to port 0xC (Respond to 68K)
	out		(0),a			; write to port 0 (Clear sound code)
	ld		sp,0xFFFC		; set stack pointer

	; call Command 01
	ld		hl,command_01
	push	hl
	retn

;==============================================================================;
; doCmd03
; Performs setup work for Command $03 (Soft Reset).

doCmd03:
	di
	xor		a
	out		(0xC),a			; write 0 to port 0xC (Respond to 68K)
	out		(0),a			; write to port 0 (Clear sound code)
	ld		sp,0xFFFC		; set stack pointer

	; call Command 03
	ld		hl,command_03
	push	hl
	retn

;==============================================================================;
; SetDefaultBanks
; Sets the default program banks. This setup treats the M1 ROM as linear space.

SetDefaultBanks:
	SetBank 0x1E,8			; Set $F000-$F7FF bank to bank $1E (30 *  2K)
	SetBank 0xE,9			; Set $E000-$EFFF bank to bank $0E (14 *  4K)
	SetBank 6,0xA			; Set $C000-$DFFF bank to bank $06 ( 6 *  8K)
	SetBank 2,0xB			; Set $8000-$BFFF bank to bank $02 ( 2 * 16K)
	ret

;==============================================================================;
; fm_Stop
; Stops all FM channels.

; "...if you're accessing the same address multiple times, you may write the
; address first and procceed to write the data register multiple times."
; - translated from YM2610 Application Manual, Section 9

fm_Stop:
	di
	push	af
	ld		a,0x28			; Slot and Key On/Off
	out		(4),a			; write to port 4 (address 1)
	rst		8				; Write delay 1 (17 cycles)
	;---------------------------------------------------;
	ld		a,0x01			; FM Channel 1
	out		(5),a			; write to port 5 (data 1)
	rst		0x10			; Write delay 2 (83 cycles)
	;---------------------------------------------------;
	ld		a,0x02			; FM Channel 2
	out		(5),a			; write to port 5 (data 1)
	rst		0x10			; Write delay 2 (83 cycles)
	;---------------------------------------------------;
	ld		a,0x05			; FM Channel 3
	out		(5),a			; write to port 5 (data 1)
	rst		0x10			; Write delay 2 (83 cycles)
	;---------------------------------------------------;
	ld		a,0x06			; FM Channel 4
	out		(5),a			; write to port 5 (data 1)
	rst		0x10			; Write delay 2 (83 cycles)
	pop		af
	ret

;==============================================================================;
; ssg_Stop
; Silences and stops all SSG channels.

ssg_Stop:
	di
	ld		de,0x0800		; SSG Channel A Volume/Mode
	write45					; write to ports 4 and 5
	;-------------------------------------------------;
	ld		de,0x0900		; SSG Channel B Volume/Mode
	write45					; write to ports 4 and 5
	;-------------------------------------------------;
	ld		de,0x0A00		; SSG Channel C Volume/Mode
	write45					; write to ports 4 and 5
	;-------------------------------------------------;
	; Disable all SSG channels... you may not want this.
	ld		de,0x073F		; Disable all SSG channels (Tone and Noise)
	write45					; write to ports 4 and 5
	ret

;==============================================================================;
; pcma_Stop
; Stops all ADPCM-A channels.

pcma_Stop:
	di
	ld		de,0x009F			; $009F		Dump all ADPCM-A channels (stop sound)
	write67

	ret

;==============================================================================;
; pcmb_Stop
; Stops the ADPCM-B channel.

pcmb_Stop:
	di
	ld		de,0x1001			; $1001		Force stop synthesis
	write45

	dec		e					; $1000		Stop ADPCM-B output
	write45
	ret

;==============================================================================;
; all of these are writes to ports 4/5:
; Set all Timer flags (0x273F)
; Clear all Timer flags (0x2700)
; Reset A/B flags, Load and Enable Timer B (0x273A)
; Reset A/B flags, Load and Enable Timer A (0x2735)
; Reset Timer A flag, Enable and Load Timers A/B (0x271F)
; Reset Timer B flag, Enable and Load Timers A/B (0x272F)

; they're also not used in this demo, so the code isn't here.

;==============================================================================;
; HandleCommand
; Handles any command that isn't already dealt with separately (e.g. $01, $03).

HandleCommand:
	ld		a,(curCommand)	; get current command
	ld		b,a				; save backup of command

	; commands $80-$87 change ADPCM-B rate.
	; The check is handled in a very simple manner for this demo program.
	jp		M,command_ChangeRate

	; command $0A: play ADPCM-A (channel 1)
	cp		SOUNDCMD_PlayA
	jp		Z,command_PlayPCMA

	; command $0B: play ADPCM-B
	cp		SOUNDCMD_PlayB
	jp		Z,command_PlayPCMB

	; command $1A: stop ADPCM-A
	cp		SOUNDCMD_StopA
	jp		Z,pcma_Stop

	; command $1B: stop ADPCM-B
	cp		SOUNDCMD_StopB
	jp		Z,pcmb_Stop

	; command $2A: loop ADPCM-A
	cp		SOUNDCMD_LoopA
	jp		Z,command_LoopPCMA

	; command $2B: loop ADPCM-B
	cp		SOUNDCMD_LoopB
	jp		Z,command_LoopPCMB

	; command $4A: no loop ADPCM-A
	cp		SOUNDCMD_NoLoopA
	jp		Z,command_NoLoopPCMA

	; command $4B: no loop ADPCM-B
	cp		SOUNDCMD_NoLoopB
	jp		Z,command_NoLoopPCMB

	; if it's not one of the above, then do nothing.
	ret

;==============================================================================;
; system command holding cell.

;------------------------------------------------------------------------------;
; command_00
; Dudley do-nothing.

command_00:
	ret

;------------------------------------------------------------------------------;
; command_01
; Handles a slot switch.

command_01:
	di
	xor		a
	out		(0xC),a			; Write 0 to port 0xC (Reply to 68K)
	out		(0),a			; Reset sound code

	call	SetDefaultBanks	; initialize banks to default config

	; (FM) turn off Left/Right, AM Sense and PM Sense
	ld		de,0xB500		; $B500: turn off for channels 1/3
	write45
	write67
	ld		de,0xB600		; $B600: turn off for channels 2/4
	write45
	write67

	; (ADPCM-A, ADPCM-B) Reset ADPCM channels
	ld		de,0x00BF		; $00BF: ADPCM-A Dump=1, all channels=1
	write67
	ld		de,0x1001		; $1001: ADPCM-B Reset=1
	write45

	; (ADPCM-A, ADPCM-B) Poke ADPCM channel flags (write 1, then 0)
	ld		de,0x1CBF		; $1CBF: Reset flags for ADPCM-A 1-6 and ADPCM-B
	write45
	ld		de,0x1C00		; $1C00: Enable flags for ADPCM-A 1-6 and ADPCM-B
	write45

	; silence FM channels
	ld		de,0x2801		; FM channel 1 (1/4)
	write45
	ld		de,0x2802		; FM channel 2 (2/4)
	write45
	ld		de,0x2805		; FM channel 5 (3/4)
	write45
	ld		de,0x2806		; FM channel 6 (4/4)
	write45

	; silence SSG channels
	ld		de,0x800		;SSG Channel A
	write45
	ld		de,0x900		;SSG Channel B
	write45
	ld		de,0xA00		;SSG Channel C
	write45

	; set up infinite loop in RAM
	ld		hl,0xFFFD
	ld		(hl),0xC3		; Set 0xFFFD = 0xC3 ($C3 is opcode for "jp")
	ld		(0xFFFE),hl		; Set 0xFFFE = 0xFFFD (making "jp $FFFD")
	ld		a,1
	out		(0xC),a			; Write 1 to port 0xC (Reply to 68K)
	jp		0xFFFD			; jump to infinite loop in RAM

;------------------------------------------------------------------------------;
; command_03
; Handles a soft reset.

command_03:
	di
	ld		a,0
	out		(0xC),a			; Write to port 0xC (Reply to 68K)
	out		(0),a			; Reset sound code
	ld		sp,0xFFFF
	jp		Start			; Go back to the top.

;------------------------------------------------------------------------------;
; command_PlayPCMA (command $0A)
; Play a sample on the first ADPCM-A channel.

command_PlayPCMA:
	di

	; reset and enable
	ld		de,0x1C01
	write45
	ld		de,0x1C00
	write45

	; volume/output ($08)
	ld		de,0x08DF
	write67

	; start addr ($10,$18)
	; samples_PCMA word 1
	ld		ix,samples_PCMA
	ld		d,0x10
	ld		a,(ix)
	ld		e,a
	write67
	ld		d,0x18
	ld		a,(ix+1)
	ld		e,a
	write67

	; end addr ($20,$28)
	; samples_PCMA word 2
	ld		d,0x20
	ld		a,(ix+2)
	ld		e,a
	write67
	ld		d,0x28
	ld		a,(ix+3)
	ld		e,a
	write67

	; play sound on channel 1
	ld		de,0x0001
	write67

	ret

;------------------------------------------------------------------------------;
; command_PlayPCMB (command $0B)
; Play an ADPCM-B sample.

command_PlayPCMB:
	di

	; don't bother trying to play back ADPCM-B on Neo-Geo CD; it won't work.
	ifnd TARGET_CD

	; reset and enable
	ld		de,0x1C80
	write45
	ld		de,0x1C00
	write45
	ld		de,0x1000
	write45

	; --left/right ($11)--
	ld		de,0x11C0
	write45

	; --volume ($1B)--
	ld		de,0x1BFF
	write45

	; --start addr ($12,$13)--
	; samples_PCMB word 1 (low first, then high)
	ld		ix,samples_PCMB
	ld		d,0x12
	ld		a,(ix)
	ld		e,a
	write45
	ld		d,0x13
	ld		a,(ix+1)
	ld		e,a
	write45

	; --end addr ($14,$15)--
	; samples_PCMB word 2 (low first, then high)
	ld		d,0x14
	ld		a,(ix+2)
	ld		e,a
	write45
	ld		d,0x15
	ld		a,(ix+3)
	ld		e,a
	write45

	; --delta-n ($19,$1A)--
	ld		d,0x19
	ld		a,(intPCMB_SampRateL)
	ld		e,a
	write45
	ld		d,0x1A
	ld		a,(intPCMB_SampRateH)
	ld		e,a
	write45

	; --main control ($10)--
	; %SxxLxxxR (S=start, L=loop, R=Reset)
	; repeat/loop flag = loopB<<4
	ld		a,(loopB)
	sla		a
	sla		a
	sla		a
	sla		a
	or		0x80		; start flag
	ld		d,0x10
	ld		e,a
	write45

	endif

.PlayPCMB_end:
	ret

;------------------------------------------------------------------------------;
; command_LoopPCMA (command $2A)
; Sets looping for ADPCM-A sample.

command_LoopPCMA:
	ld		a,1
	ld		(loopA),a
	ret

;------------------------------------------------------------------------------;
; command_LoopPCMB (command $2B)
; Sets looping for ADPCM-B sample.

command_LoopPCMB:
	ld		a,1
	ld		(loopB),a
	ret

;------------------------------------------------------------------------------;
; command_NoLoopPCMA (command $4A)
; Disables looping for ADPCM-A sample.

command_NoLoopPCMA:
	xor		a
	ld		(loopA),a
	ret

;------------------------------------------------------------------------------;
; command_NoLoopPCMB (command $4B)
; Disables looping for ADPCM-B sample.

command_NoLoopPCMB:
	xor		a
	ld		(loopB),a
	ret

;------------------------------------------------------------------------------;
; Sample Rates in this demo:
; 11025 | delta-n: 0x32DA (13018) | rate: 1.00x | $80 | (native rate (encoded))
; 16538 | delta-n: 0x4C49 (19529) | rate: ~1.5x | $81 |
; 22050 | delta-n: 0x65B5 (26037) | rate: 2.00x | $82 |
; 27563 | delta-n: 0x7F23 (32547) | rate: ~2.5x | $83 |
; 33075 | delta-n: 0x9890 (39056) | rate: 3.00x | $84 |
; 38588 | delta-n: 0xB1FE (45566) | rate: ~3.5x | $85 |
; 44100 | delta-n: 0xCB6B (52075) | rate: 4.00x | $86 | (original sample's rate)
; 55125 | delta-n: 0xFE45 (65093) | rate: 5.00x | $87 |

; low byte (port 0x19)
tbl_RatesPCMB_L:
	db 0xDA
	db 0x49
	db 0xB5
	db 0x23
	db 0x90
	db 0xFE
	db 0x6B
	db 0x45

; high byte (port 0x1A)
tbl_RatesPCMB_H:
	db 0x32
	db 0x4C
	db 0x65
	db 0x7F
	db 0x98
	db 0xB1
	db 0xCB
	db 0xFE

; command_ChangeRate (commands $80-$87)
; Changes rate of ADPCM-B sample.

command_ChangeRate:
	; mask command number w/0x07 to get new rate index
	and		0x07
	; index into tbl_RatesPCMB_L and tbl_RatesPCMB_H
	ld		c,a
	xor		a
	ld		b,a

	; set intPCMB_SampRateL
	ld		hl,tbl_RatesPCMB_L
	add		hl,bc
	ld		a,(hl)
	ld		(intPCMB_SampRateL),a

	; set intPCMB_SampRateH
	ld		hl,tbl_RatesPCMB_H
	add		hl,bc
	ld		a,(hl)
	ld		(intPCMB_SampRateH),a

	; change sample rate as it's playing, too, just in case
	ld		d,0x19
	ld		a,(intPCMB_SampRateL)
	ld		e,a
	write45
	ld		d,0x1A
	ld		a,(intPCMB_SampRateH)
	ld		e,a
	write45

	ret

;==============================================================================;
; junior samples jr.
	ifd TARGET_CD
		include "samples_cd.inc"
	else
		include "samples_cart.inc"
	endif

;==============================================================================;
; Z80 ram
	include "soundram.inc"
